"""Setup module for aiocomelit."""

# This is a shim to allow GitHub to detect the package
# Build metadata is in pyproject.toml
# Taken from https://github.com/Textualize/rich

import setuptools

if __name__ == "__main__":
    setuptools.setup(name="aiocomelit")
